;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x45fddbe08ace0c279b1e80905d7da374 . "./code.cg2js/cg-common-control.fasl")
(#x2c2ab23d9d07951ba06ff971b3f0332b . "./code.cg2js/cg-header-control.fasl")
(#xc1a2129caed15a33448ceff0461b7ae7 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xffdf0323667b5cc7279034950e563898 . "./code.cg2js/cg-yes-no-string.fasl")
(#x4db389ee5c479383d96650054b306f58 . "./code.cg2js/cg-clipboard.fasl")
(#x0ebc76f5e7a563b5aeaa774a9b9685bd . "./code.cg2js/cg-dde-utils.fasl")
(#x2c9769a97a1801fdc66cd204fd69b657 . "./code.cg2js/cg-editable-text.fasl")
(#x2bbaf9d892681f97c07b0ced54740b7f . "./code.cg2js/cg-text-or-combo.fasl")
(#x1e75051e1ad837ee66ab523dd45251f1 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0babf6b488667ce67375e95f07e478e8 . "./code.cg2js/cg-graphics-context.fasl")
(#xc2c273d4b50d551bc492d43c87013146 . "./code.cg2js/cg-choose-printer.fasl")
(#x498a0222852ec83dada6abfcc311301e . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xfac4e6323a11c8113e3da6eb480ea0be . "./code.cg2js/cg-edit-in-place.fasl")
(#x81effacf262dcbcfc305585f50cc3294 . "./code.cg2js/cg-split-bar.fasl")
(#x4c13778df0c400a309a83ee007891675 . "./code.cg2js/cg-timer.fasl")
(#xa6c29ac788c7f32c97ad5d624bbb4566 . "./code.cg2js/cg-progress-indicator.fasl")
(#x86a318fd77c923db841ede66cbe697b4 . "./code.cg2js/cg-object-editor.fasl")
(#xa53ae8039432cbda0659ee2aed151914 . "./code.cg2js/cg-fill-texture.fasl")
(#x88786d707fc69b980b920425a9dc6fe7 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x4dc8fd6c0b6fc86f158f1c897864eacc . "./code.cg2js/cg-os-window.fasl")
(#xec359c6b723f18bd91fc13d8c5e9f061 . "./code.cg2js/cg-color-dialog.fasl")
(#x341060d78bb859be3376447a74e33927 . "./code.cg2js/cg-combo-box.fasl")
(#x5ab36335d9e7d2189489692c400c5cb3 . "./code.cg2js/cg-directory-dialog.fasl")
(#x1a8dced678014095a1b5852c21e878a0 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xbbb8b90e38980e7dfebc94b77a9f283e . "./code.cg2js/cg-tooltip.fasl")
(#xdd9622bc8ef8556eb2f04c056a443005 . "./code.cg2js/cg-lettered-menu.fasl")
(#xf836f9e23a0be300351e315f3e5b8d81 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xc1eb9c6a07397113687160d35e2499ee . "./code.cg2js/cg-pixmap.fasl")
(#x1856125f4061c5c75dfb9ae3648e906a . "./code.cg2js/cg.fasl")
(#x0a9c6b7ef960834ad796fd1e63f753ba . "./code.cg2js/cg-class-slot-grid.fasl")
(#x4f62e2c1cc5a1def5c0ca12c6bf36dd7 . "./code.cg2js/cg-template-string.fasl")
(#x523c3c15fd34df7e158a46647f7fbfae . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x7aac0528ea9e950704df79e088a58f42 . "./code.cg2js/cg-check-box.fasl")
(#x72b99e27845efd52449d5794f7ea57b7 . "./code.cg2js/cg-toggling-widget.fasl")
(#xa54eb133494201def17e1e8095b2d232 . "./code.cg2js/cg-radio-button.fasl")
(#x5c39f3c005b47364c7d3cd5badcca4fb . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x1c189899930bad925b05bae367ffe86b . "./code.cg2js/cg-scaling-stream.fasl")
(#xbe16d03ea03f71c9f52806b72ca3222a . "./code.cg2js/cg-multi-picture-button.fasl")
(#x01516b99db37014d86ed54a270c78d78 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x6e9dc72c70726299b583aa6cf63ea657 . "./code.cg2js/cg-grid-widget.fasl")
(#xa97da332b89579f2254a2afdf0a60b95 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xaff9124f4bf7bab713a9acb6f2609d50 . "./code.cg2js/cg-item-list.fasl")
(#xccfdefc369e43ae55c066de50866adfb . "./code.cg2js/cg-paren-matching.fasl")
(#x0cc7a356339177f426c0eae759df4294 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xd40b25900ad68beacac8cb1be090eabb . "./code.cg2js/cg-html-dialog.fasl")
(#x95403083c97f8a795bbb1453658c030f . "./code.cg2js/cg-pixmap-widget.fasl")
(#xa8ba3be2ae045a421ae2874c1ac11a98 . "./code.cg2js/cg-project-window.fasl")
(#x690250fd83bf16344c34528bfc27352c . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x369937571e92b22de5224f61bf40bace . "./code.cg2js/cg-toolbar.fasl")
(#x47300b1f2a0cdb3725fd1e211f5c68e4 . "./code.cg2js/cg-caret.fasl")
(#x49ff80bccb9e279627d6c285d3779280 . "./code.cg2js/cg-scroll-bar.fasl")
(#x9503b329564848b0a25cb238960d728c . "./code.cg2js/cg-web-browser.fasl")
(#x11281932f265288c4c6d0ca8d8fd8777 . "./code.cg2js/cg-dropping-outline.fasl")
(#xd72bfc4d71d0591a4503f0e0ab68e21b . "./code.cg2js/cg-font-dialog.fasl")
(#xf10d77d7359bb08e3994898dc9d57099 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe90c9f85556cdc68fad4c8641495e8cf . "./code.cg2js/cg-yes-no-list.fasl")
(#x876681c517a8e4a291e20c7dfb90166b . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9383400a649d3bdb3c572f932a26f165 . "./code.cg2js/cg-lamp.fasl")
(#x1291f34a9b3a9c90a730e0da94cb2e38 . "./code.cg2js/cg-drawable.fasl")
(#xc48cde60535f93fed26950831d98ab0b . "./code.cg2js/cg-choice-list.fasl")
(#x7792fdfe930a5e27874b587ca46d7f98 . "./code.cg2js/cg-curve.fasl")
(#xac96b0ac660bd0c8b491adf926695bcc . "./code.cg2js/cg-base.fasl")
(#xacd294e4868851d96c6d5668a45e46fd . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x8235bde97be9a280f2594ff2886b75d4 . "./code.cg2js/cg-printing.fasl")
(#x1b24ec7ed9722ca59c02377fc5921fae . "./code.cg2js/cg-chart-or-plot.fasl")
(#x862c76bd922239ba144a7211a70739fe . "./code.cg2js/cg-hotspot.fasl")
(#x673338c88fab98df7ed91896e9767d6b . "./code.cg2js/cg-object-editor-layout.fasl")
(#x8e8b362b1ed5bcc9465f83d1004430bd . "./code.cg2js/cg-utility-dialog.fasl")
(#x644b534f44b36d99054b3b61086034aa . "./code.cg2js/cg-shortcut-menu.fasl")
(#x3b238a0ce0d3d72c630fc04d3b0d3697 . "./code.cg2js/cg-comtab.fasl")
(#xf3c88d4adc0979d6579f7a86045febd3 . "./code.cg2js/cg-os-widget.fasl")
(#x4a98948a80ab51e0363b8aac54f21bd4 . "./code.cg2js/cg-icon.fasl")
(#x3ae857ad87e5362d868eb2990f9e738e . "./code.cg2js/cg-dev.fasl")
(#x28d176feae696581dfb48604cf70f290 . "./code.cg2js/cg-calendar.fasl")
(#x422661234e734b75a8eb9d0537dec23c . "./code.cg2js/cg-group-box.fasl")
(#x51faa814929631bc3ed78a3c558393e9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xd407ef86cf40fd25f572c3e2922440d0 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xacc3d670b7ae63f640eaf3fe2c088dd9 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xa6a34d01f601ae2bbb95f266eb568318 . "./code.cg2js/cg-nodes.fasl")
(#x723bdade25325d3bc76841dc60dd8fa2 . "./code.cg2js/cg-intersect.fasl")
(#x51adead9b976ed1632c82cef640e407e . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x13cd2ddccac4b82f1d3af401dd16c2d6 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xbaa3fdb6414c82aac791cdad00b4be2e . "./code.cg2js/cg-class-support.fasl")
(#x07025d4aaa0f547655a50ce561b642ca . "./code.cg2js/cg-dialog-item.fasl")
(#x6810b3f3b37fa3493cd07cfd5c4fdbcd . "./code.cg2js/cg-plot-widget.fasl")
(#x22d554f93e22b86c0c65cded2eb02ec0 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe4304c4bdfecfddca6fb2f0736b35849 . "./code.cg2js/cg-up-down-control.fasl")
(#xf50fc6350ef9a22c9755d90452800b56 . "./code.cg2js/cg-rich-edit.fasl")
(#x54ba6e5a411340fc51bb2f93e1fb80b2 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x84bee838e48e6642c62f3eaa6bd89104 . "./code.cg2js/cg-status-bar.fasl")
(#x0eaa219ad5acd0c682e132a4c8e9d5ff . "./code.cg2js/cg-class-grid.fasl")
(#xf2d477421c415f88a9ba20d3ccfd08a4 . "./code.cg2js/cg-wrap-string.fasl")
(#xb7375eb66f3ffaddf422ea2e59e4a642 . "./code.cg2js/cg-layout.fasl")
(#x957a5607aa7e1f59e6c44923f50a2ef7 . "./code.cg2js/cg-lisp-widget.fasl")
(#x50e19417df8fef5cc618928f98b06be6 . "./code.cg2js/cg-button.fasl")
(#x2c669c79ba47dcd1be3d3f0e9954629c . "./code.cg2js/cg-string-dialog.fasl")
(#xbdbcef2bb7ae3a02f3b699d5713c4c3e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xab6741cb4ea696d9bd5d49544f69fec0 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xe968398d854cdf4db5c715211e466fba . "./code.cg2js/cg-bitmap-stream.fasl")
(#xda15326bca62d46440134758cd62edd7 . "./code.cg2js/cg-static-text.fasl")
(#x0ab4f3bc999cf7988e487057bf8b0d6a . "./code.cg2js/cg-outline.fasl")
(#x731aed046a5506740e2cf565578353f4 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xaad21937929e01696c1c3f59f8649342 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xd1ab2b801655a295ce6ca2f8688cb4ce . "./code.cg2js/cg-picture-widget.fasl")
(#x058056c6e246c813bb25083f4d53248b . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x80628c5c35555bb08ee6af2ccbc7005a . "./code.cg2js/cg-chart-widget.fasl")
(#xd53b0adc1fc95db453738ea81778c888 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x3674a5a3228253452f1a301603bf972c . "./code.cg2js/cg-lisp-text.fasl")
(#x3eccdd46a18768850d053365372ee4c1 . "./code.cg2js/cg-palette.fasl")
(#xd1173aacfa1e38cc46b32d55db2731d9 . "./code.cg2js/cg-property.fasl")
(#x6c48751bda086bdf0a2b2637546a1894 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x4e7da2bfe9ad89529683ce6a7692806a . "./code.cg2js/cg-file-dialog.fasl")
(#x62a0489a5be2690f9c2c2c38406824fb . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xf0e2cd93de6adf2e67d7a27d2b0e6a10 . "./code.cg2js/cg-get-pixmap.fasl")
(#xdcf3b45559be72581039b45954be4976 . "./code.cg2js/cg-tray.fasl")
(#x66d3993e02aa9cfd3a7dd54b98cf8ac1 . "./code.cg2js/cg-acache.fasl")
(#xf782efd998404663cc33471709cb3422 . "./code.cg2js/cg-message-dialog.fasl")
(#xfb9eba9d3721c55576f26efe9bd299a5 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x29f225aa556d91462baa802ce68c8b4b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xd27e45fa43594c37fe72f4a0a76c6235 . "./code.cg2js/cg-tab-control.fasl")
(#x388c9865908c7b4d19cf72b8b98bb808 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xab925f5c9088cc7eabe608c3bb81d285 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x15260925b638a0a6026703cdd0b1e9d0 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xf885cb3a8206fb88cc85f75caa14c8c9 . "./code.cg2js/cg-menu.fasl")
(#x5c0f82acef15dcd514badf1686bf9cd2 . "./code.cg2js/cg-get-position.fasl")
(#x79796b2ef77406b871f8069fdab7953f . "./code.cg2js/cg-trackbar.fasl")
(#x693e2b8004f79ffec7fbb5484ba9c361 . "./code.cg2js/cg-selected-object.fasl")
(#x75e31d187f269d4c431bdf481391c329 . "./code.cg2js/cg-list-view.fasl")
(#x795ac84c224d806fd01b2424e1865969 . "./code.cg2js/cg-html-widget.fasl")
(#x3484106ea62adad7449306512222f10a . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe7b90684798a1e2c5b454e27b37c78ca . "./code.cg2js/cg-text-widget.fasl")
(#x0401bf365d83ce1e40c9f225cc19590f . "./code.idejs/ide-base.fasl")
(#x5e988497578f58d2041fa3222afcd571 . "./code.idejs/ide-process-browser.fasl")
(#x6ba7222e4aec438afe39514869a099c3 . "./code.idejs/ide-patch-dialog.fasl")
(#xd3ffbbb3a213df62f0d198834414ac94 . "./code.idejs/ide-keyshort.fasl")
(#x78616d67eb26dffad17bf35252ba869f . "./code.idejs/ide-window-list.fasl")
(#x6211c5a6d81e52769b7de6c0b0ad2ad7 . "./code.idejs/ide.fasl")
(#xd7174c1c29d282a92e5acbab5e3af65e . "./code.idejs/ide-editor.fasl")
(#x05e15c9de4bb0a180448b1278ca107f7 . "./code.idejs/ide-clipboard.fasl")
(#xa07cb786bc9ed3be2fcb71c00deb30d1 . "./code.idejs/ide-trace.fasl")
(#xca79ebccee3f6fe4e765ec75efc2a6f0 . "./code.idejs/ide-class-browser.fasl")
(#x9a0c8df48385273a1712cc611d199bae . "./code.idejs/ide-grapher.fasl")
(#xdbcfe2f4e297ed07afd9d75ea5a9d4d7 . "./code.idejs/ide-apropos.fasl")
(#x70ac74945142c75a780b4dc18d66eae3 . "./code.idejs/ide-navigator.fasl")
(#xca38dc8246e4c1fc5d092438d38c05bc . "./code.idejs/ide-find-definitions.fasl")
(#x61ee15fb8b112463a7e140e91216e6f5 . "./code.idejs/ide-project.fasl")
(#x90298a499e7daa1fc31de905330b3d65 . "./code.idejs/ide-class-grapher.fasl")
(#x0f71fb7c5e11b86ac5a40fde0c826ea6 . "./code.idejs/ide-acache.fasl")
(#xf48166057b42ec7cb8911b44bec7b599 . "./code.idejs/ide-find-in-files.fasl")
(#x0b9ef5d85fb9b4eeea8c041e5074c30f . "./code.idejs/ide-inspector.fasl")
(#x533c4ee8a77d9b0d3b762c278aa1a3f6 . "./code.idejs/ide-intro.fasl")
(#xfd6396076f7ba2c8f7405b8e47263428 . "./code.idejs/ide-package-list.fasl")
(#xe545a26f3406e8aff2dfbdd3889e7260 . "./code.idejs/ide-dev.fasl")
(#x029ab71e65e55950480f28e9e3009cc4 . "./code.idejs/ide-form.fasl")
(#xe396744af4d34e05e734175c54f8cafc . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
